'use strict';

/**
 * esnext-coverage reporter module.
 * @module esnext-coverage-reporter
 */

/* eslint-disable no-console */

/**
 * EsnextCoverageReporter.
 * @constructor
 * @param {Object} config - esnext-coverage config obtained from karma conf file.
 */
module.exports = function EsnextCoverageReporter(config) {

  var fs = require('fs');
  var path = require('path');
  var Promise = require('promise');
  var mkdirp = Promise.denodeify(require('mkdirp'));

  var _require = require('chalk');

  var red = _require.red;
  var stripColor = _require.stripColor;

  var requireFormatter = require('./require-formatter');
  var determineThresholds = require('./determine-thresholds');

  var writeFile = Promise.denodeify(fs.writeFile);

  /**
   * Asynchronous tasks (i.e. writing to filesystem).
   * @type {[Promise]}
   */
  var asyncTasks = [];

  /**
   * Reporter configuration.
   * @type {Object}
   * @property {String} dir - relative path of output directory
   * @property {Array} formatters - list of esnext-coverage formatters to use
   * @property {Object} thresholds - code coverage thresholds
   */
  var reporterConfig = config.esnextCoverageReporter || {};

  /**
   * Code coverage thresholds.
   * @type {Object}
   * @property {Object} global - Average across multiple files.
   * @property {Object} local - Local per-file coverage thresholds.
   */
  var thresholds = determineThresholds(reporterConfig.thresholds);

  /**
   * Coverage result formatters.
   * @type {[Object]}
   */
  var reporters = (reporterConfig.reporters || []).map(function (formatterConfig) {
    formatterConfig.formatter = requireFormatter(formatterConfig.formatter);
    return formatterConfig;
  });

  /**
   * Browser complete event handler.
   * @param {Object} browser - Information about the browser.
   * @param {Object} result - Runner results.
   * @param {Object} result.coverage - esnext-coverage coverage object.
   * @returns {undefined} Nothing is returned.
   */
  this.onBrowserComplete = function (browser, _ref) {
    var coverage = _ref.coverage;

    // In case of errors encountered prior to esnext-coverage execution
    // the coverage object will be `undefined` and there's nothing
    // we can do about it except aborting:
    if (!coverage) {
      console.error(red('\nesnext-coverage was unable to generate code coverage\n'));
      return;
    }

    reporters.forEach(function (formatterConfig) {
      var formattedText = formatterConfig.formatter(coverage, {
        environment: browser,
        thresholds: thresholds
      });

      if (formatterConfig.console) {
        // Dump formatted text to console,
        // and surround with single line margins:
        console.log('\n' + formattedText + '\n');
      }

      if (formatterConfig.outFile) {
        (function () {
          var fullpath = config.browsers.length > 1 ? path.join(reporterConfig.dir, browser.name, formatterConfig.outFile) : path.join(reporterConfig.dir, formatterConfig.outFile);

          asyncTasks.push(mkdirp(path.dirname(fullpath)).then(function () {
            return writeFile(fullpath, stripColor(formattedText));
          }));
        })();
      }
    });
  };

  /**
   * Karma exit event handler.
   * @param {Function} done - Karma completion callback.
   * @returns {undefined} Nothing is returned.
   */
  this.onExit = function (done) {
    // Karma runner will complete before we finish writing
    // report files to the filesystem, so let's wait for
    // all those asynchronous tasks to finish before
    // declaring that we're done:
    Promise.all(asyncTasks).finally(function () {
      return done();
    });
  };
};

/**
 * Karma-specific dependency injection.
 * @type {[String]}
 */
module.exports.$inject = ['config'];